var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { join } from 'node:path';
const dashboardPath = 'astro-netlify-cms/cms';
const AdminPage = ({ adminPath, assets, config, dashboardPath, previewStyles = [], projectRoot, }) => {
    const imports = [];
    const styles = [];
    if (assets) {
        for (const [name, filename] of assets) {
            // At build time, pass the path of built styles straight to Netlify CMS.
            if (name.startsWith('style__')) {
                styles.push(`['/${filename}']`);
            }
            else {
                imports.push(`import ${name} from '/${filename}';`);
            }
        }
    }
    else {
        imports.push(`import cms from '${dashboardPath}';`);
    }
    previewStyles.forEach((entry, index) => {
        if (!Array.isArray(entry))
            entry = [entry];
        const [style, opts] = entry;
        if ((opts === null || opts === void 0 ? void 0 : opts.raw) || style.startsWith('http')) {
            styles.push(JSON.stringify([style, opts]));
        }
        else if (!assets) {
            const name = `style__${index}`;
            imports.push(`import ${name} from '${join(projectRoot, style)}';`);
            styles.push(`[${name}, { raw: true }]`);
        }
    });
    return `<html lang="en">
  <head>
  <title>Content Manager</title>
  <meta
  name="description"
  content="Admin dahsboard for managing website content"
  />
  <script type="module">
  ${imports.join('\n')}
  cms({
    adminPath: '${adminPath}',
    config: JSON.parse('${JSON.stringify(config)}'),
    previewStyles: [${styles.join(',')}],
  });
  </script>
  </head>
  <body></body>
  </html>
  `;
};
export default function AdminDashboardPlugin({ adminPath, config, previewStyles, }) {
    if (!adminPath.startsWith('/')) {
        throw new Error('`adminPath` option must be a root-relative pathname, starting with "/", got "' +
            adminPath +
            '"');
    }
    if (adminPath.endsWith('/')) {
        adminPath = adminPath.slice(0, -1);
    }
    let projectRoot;
    let importMap;
    return {
        name: 'vite-plugin-netlify-cms-admin-dashboard',
        configResolved({ root }) {
            projectRoot = root;
        },
        options(options) {
            let { input } = options;
            if (input &&
                (!Array.isArray(input) ||
                    !input.includes('@astrojs-pages-virtual-entry'))) {
                if (!Array.isArray(input) && typeof input !== 'object')
                    input = [input];
                importMap = generateImportMap({
                    dashboardPath,
                    previewStyles,
                    projectRoot,
                });
                input = Object.assign(Object.assign({}, input), importMap);
            }
            return Object.assign(Object.assign({}, options), { input });
        },
        configureServer({ transformIndexHtml, middlewares }) {
            middlewares.use((req, res, next) => __awaiter(this, void 0, void 0, function* () {
                if (req.url === adminPath || req.url === adminPath + '/') {
                    const adminPage = yield transformIndexHtml(adminPath, AdminPage({
                        adminPath,
                        config,
                        dashboardPath,
                        previewStyles,
                        projectRoot,
                    }));
                    res.end(adminPage);
                }
                else {
                    next();
                }
            }));
        },
        generateBundle(options, bundle) {
            const dashboardChunk = Object.values(bundle).find(({ name }) => name === 'cms');
            if (!dashboardChunk)
                return;
            this.emitFile({
                type: 'asset',
                fileName: adminPath.replace(/((^\/)|(\/$))/g, '') + '/index.html',
                source: AdminPage({
                    adminPath,
                    assets: collectBundleAssets(bundle, importMap),
                    config,
                    previewStyles,
                    projectRoot,
                }),
            });
        },
    };
}
/**
 * Generate a map of asset IDs to paths where they can be imported.
 * @returns An object like:
 * ```js
 * {
 *   cms: 'astro-netlify-cms/cms.ts',
 *   style__0: '/path/to/project/dir/path/to/style.css',
 * }
 * ```
 */
function generateImportMap({ dashboardPath, previewStyles, projectRoot, }) {
    const imports = { cms: dashboardPath };
    previewStyles.forEach((entry, index) => {
        if (!Array.isArray(entry))
            entry = [entry];
        const [style, opts] = entry;
        if ((opts === null || opts === void 0 ? void 0 : opts.raw) || style.startsWith('http'))
            return;
        imports[`style__${index}`] = join(projectRoot, style);
    });
    return imports;
}
/**
 * Extract assets in the import map from Rollup’s bundle.
 */
function collectBundleAssets(bundle, importMap) {
    const stripExtension = (filename) => filename.split('.')[0];
    return Object.values(bundle)
        .filter(({ name, fileName }) => !!name &&
        stripExtension(name) in importMap &&
        // Filter out non-CSS `style__` assets (a tiny JS file seems to get built for each)
        (!name.startsWith('style__') || fileName.endsWith('.css')))
        .map(({ name, fileName }) => [stripExtension(name), fileName]);
}
